#include <stdio.h>
#include <stdlib.h>
#include "cpucycles.h"
#include "crypto_kem.h"

#define TIMINGS 5

long long t[TIMINGS + 1];

void printtimings(const char *s)
{
  long long i;
  long long j;
  long long above;
  long long below;

  printf("%lld %s",(long long) crypto_kem_PUBLICKEYBYTES,s);
  for (i = 0;i < TIMINGS;++i) t[i] = t[i + 1] - t[i];
  for (j = 0;j < TIMINGS;++j) {
    above = below = 0;
    for (i = 0;i < TIMINGS;++i) if (t[i] < t[j]) ++below;
    for (i = 0;i < TIMINGS;++i) if (t[i] > t[j]) ++above;
    if (below <= TIMINGS/2 && above <= TIMINGS/2) {
      printf(" %lld (%lf)",t[j],t[j] / (1.0 * crypto_kem_PUBLICKEYBYTES));
      break;
    }
  }

  for (i = 0;i < TIMINGS;++i) printf(" %lld",t[i]);
  printf("\n");
}

int main()
{
  unsigned char *sk = malloc(crypto_kem_SECRETKEYBYTES);
  unsigned char *pk = malloc(crypto_kem_PUBLICKEYBYTES);
  unsigned char *c = malloc(crypto_kem_CIPHERTEXTBYTES);
  unsigned char *k = malloc(crypto_kem_BYTES);
  long long i;

  if (!sk) abort();
  if (!pk) abort();
  if (!c) abort();
  if (!k) abort();

  t[0] = cpucycles();
  crypto_kem_keypair(pk,sk);
  t[1] = cpucycles();
  t[1] -= t[0];
  printf("%lld keypair %lld (%lf)\n",(long long) crypto_kem_PUBLICKEYBYTES
    ,t[1],t[1] / (1.0 * crypto_kem_PUBLICKEYBYTES));

  for (i = 0;i < TIMINGS + 1;++i) {
    t[i] = cpucycles();
    crypto_kem_enc(c,k,pk);
  }
  printtimings("enc");

  for (i = 0;i < TIMINGS + 1;++i) {
    t[i] = cpucycles();
    crypto_kem_dec(k,c,sk);
  }
  printtimings("dec");

  return 0;
}
